@@ -195,9 +195,17 @@ extern "C"
 #define G400x600x16M 143
 #define G400x600x16M32 144
 
-#define __GLASTMODE G400x600x16M32
+#define G400x300x256X 145
+
+#define G320x200x256V 146
+
+#define __GLASTMODE G320x200x256V
 #define GLASTMODE vga_lastmodenumber()
 
+#define IS_IN_STANDARD_VGA_DRIVER(mode) ( \
+	((mode) < G640x480x256) || ((mode) == G720x348x2) || \
+	( ((mode) >= G400x300x256X) && ((mode) <= G400x300x256X) ) )
+
     extern int vga_version;
 
     extern int vga_setmode(int mode);
@@ -247,7 +255,8 @@ extern "C"
 #define IS_MODEX 4		/* ModeX style 256 colors */
 #define IS_DYNAMICMODE 8	/* Dynamic defined mode */
 #define CAPABLE_LINEAR 16	/* Can go to linear addressing mode. */
-#define IS_LINEAR 32		/* Linear addressing enabled. */
+#define IS_LINEAR 32		/* Linear addressing can be used. */
+#define LINEAR_MODE 512		/* Linear mode is enabled */
 #define EXT_INFO_AVAILABLE 64	/* Returned modeinfo contains valid extended fields */
 #define RGB_MISORDERED 128	/* Mach32 32bpp uses 0BGR instead of BGR0. */
     /* As of this version 1.25 also used to signal if real RGB
@@ -280,6 +289,14 @@ extern "C"
 	/* depends from actual driver/chiptype.. etc. */
     } vga_modeinfo;
 
+    typedef struct {
+        int version;
+        int size;
+        int chipset;
+        int physmem;
+    } vga_cardinfo;
+
+    extern vga_cardinfo *vga_getcardinfo(void);
     extern vga_modeinfo *vga_getmodeinfo(int mode);
     extern int vga_getdefaultmode(void);
     extern int vga_getcurrentmode(void);
@@ -302,6 +319,7 @@ extern "C"
     extern void vga_disabledriverreport(void);
     extern int vga_setmodeX(void);
     extern int vga_init(void);	/* Used to return void in svgalib <= 1.12. */
+    extern int vga_initf(int);	
     extern int vga_getmousetype(void);
     extern int vga_getmonitortype(void);
     extern void vga_setmousesupport(int s);
@@ -353,7 +371,18 @@ extern "C"
 #define SIS		24
 #define I740		25
 #define NEOMAGIC	26
-
+#define LAGUNA		27
+#define FBDEV		28
+#define G400		29
+#define R128		30
+#define SAVAGE		31
+#define MILLENNIUM	32
+#define I810		33
+#define TRIDENT		34
+#define RENDITION 	35
+#define G450C2		36
+#define PM2		37
+    
     /* Hor. sync: */
 #define MON640_60	0	/* 31.5 KHz (standard VGA) */
 #define MON800_56	1	/* 35.1 KHz (old SVGA) */
@@ -388,6 +417,9 @@ extern "C"
     extern void vga_selectcursor(int);
     extern void vga_setcursorimage(int, int, int, int, unsigned char *);
 
+    extern int vga_setcrtcregs(unsigned char *);
+    extern int vga_getcrtcregs(unsigned char *);
+
     extern int vga_addtiming(int pixelClock,
        			      int HDisplay,		
                               int HSyncStart,
@@ -426,6 +458,8 @@ extern "C"
 
    extern int vga_guesstiming(int x, int y, int clue, int arg);
 
+   extern void vga_dpms(int mode);
+
 /* Valid values for what in vga_ext_set: */
 #define VGA_EXT_AVAILABLE	0	/* supported flags */
 #define VGA_EXT_SET		1	/* set flag(s) */
