@@ -56,7 +56,7 @@ static void banshee_getmodeinfo(int mode, vga_modeinfo *modeinfo)
     if (modeinfo->bytesperpixel >= 1) {
 	if(banshee_linear_base)modeinfo->flags |= CAPABLE_LINEAR;
         if (__svgalib_banshee_inlinearmode())
-	    modeinfo->flags |= IS_LINEAR;
+	    modeinfo->flags |= IS_LINEAR | LINEAR_MODE;
     }
 }
 
@@ -69,7 +69,7 @@ typedef struct {
                 hwCurPatAddr, hwCurLoc, hwCurC0, hwCurC1;
 } *HWRecPtr;
 
-static int banshee_saveregs(unsigned char regs[])
+static int banshee_saveregs(uint8_t regs[])
 { 
   HWRecPtr save;
 
@@ -99,7 +99,7 @@ static int banshee_saveregs(unsigned char regs[])
 
 /* Set chipset-specific registers */
 
-static void banshee_setregs(const unsigned char regs[], int mode)
+static void banshee_setregs(const uint8_t regs[], int mode)
 {  
     HWRecPtr restore;
 
@@ -170,7 +170,7 @@ static int CalcPLL(int freq, int isBanshee);
 
 /* Local, called by banshee_setmode(). */
 
-static void banshee_initializemode(unsigned char *moderegs,
+static void banshee_initializemode(uint8_t *moderegs,
 			    ModeTiming * modetiming, ModeInfo * modeinfo, int mode)
 { /* long k; */
 
@@ -282,7 +282,7 @@ return ;
 
 static int banshee_setmode(int mode, int prv_mode)
 {
-    unsigned char *moderegs;
+    uint8_t *moderegs;
     ModeTiming modetiming;
     ModeInfo *modeinfo;
 
@@ -537,6 +537,8 @@ static int banshee_init(int force, int par1, int par2)
 	fprintf(stderr,"Using Banshee / Voodoo3 driver, %iKB.\n",banshee_memory);
     }
     
+	__svgalib_modeinfo_linearset |= IS_LINEAR;
+	
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = banshee_memory;
     cardspecs->maxPixelClock4bpp = 0;
