@@ -221,7 +221,7 @@ static void cirrus_getmodeinfo(int mode, vga_modeinfo * modeinfo)
     if (modeinfo->bytesperpixel >= 1) {
 	modeinfo->flags |= CAPABLE_LINEAR;
 	if (__svgalib_cirrus_inlinearmode())
-	    modeinfo->flags |= IS_LINEAR;
+	    modeinfo->flags |= IS_LINEAR | LINEAR_MODE;
     }
 }
 
@@ -819,6 +819,7 @@ static int cirrus_linear(int op, int param)
 	     * linear addressing is enabled when looking at 64MB, causing a
 	     * clash between video card and system memory at 14MB.
 	     */
+#if 0
 	    if (__svgalib_physmem() <= 13 * 1024 * 1024) {
 		if (param == 1)
 		    return 0x4000000;	/* 64MB */
@@ -829,6 +830,7 @@ static int cirrus_linear(int op, int param)
 		if (param == 4)
 		    return 0x3E00000;	/* 62MB */
 	    }
+#endif
 	    return -1;
 	}
     }
@@ -1066,14 +1068,9 @@ static int cirrus_init(int force, int par1, int par2)
        i=(__svgalib_inSR(0x17)>>3)&7;
        if(i==4) {
           unsigned long buf[64];
-          int found, _ioperm=0;
+          int found;
          
-          if(getenv("IOPERM")==NULL) {
-             _ioperm=1;
-             iopl(3);
-          };
           found=__svgalib_pci_find_vendor_vga(0x1013,buf,0);
-          if(_ioperm)iopl(0);
           if(!found)cirrus_pci_linear=buf[4]&0xffff0000;
        };
     };
@@ -1425,19 +1422,15 @@ static int cirrus_init(int force, int par1, int par2)
 #define MMIOROP			0x1A
 #define MMIOBLTSTATUS		0x40
 
-#define MMIOSETDESTADDR(addr) \
-  *(unsigned int *)(MMIO_POINTER + MMIODESTADDR) = addr;
+#define MMIOSETDESTADDR(addr) v_writel(addr, MMIODESTADDR);
 
-#define MMIOSETSRCADDR(addr) \
-  *(unsigned int *)(MMIO_POINTER + MMIOSRCADDR) = addr;
+#define MMIOSETSRCADDR(addr) v_writel(addr, MMIOSRCADDR);
 
 /* Pitch: the 5426 goes up to 4095, the 5434 can do 8191. */
 
-#define MMIOSETDESTPITCH(pitch) \
-  *(unsigned short *)(MMIO_POINTER + MMIODESTPITCH) = pitch;
+#define MMIOSETDESTPITCH(pitch) v_writel(pitch, MMIODESTPITCH);
 
-#define MMIOSETSRCPITCH(pitch) \
-  *(unsigned short *)(MMIO_POINTER + MMIOSRCPITCH) = pitch;
+#define MMIOSETSRCPITCH(pitch) v_writel(pitch, MMIOSRCPITCH);
 
 /* Width: the 5426 goes up to 2048, the 5434 can do 8192. */
 
@@ -1462,7 +1455,7 @@ static int cirrus_init(int force, int par1, int par2)
   *(unsigned char *)(MMIO_POINTER + MMIOBLTSTATUS) |= 0x02;
 
 #define MMIOBLTBUSY(s) \
-  s = *(volatile unsigned char *)(MMIO_POINTER + MMIOBLTSTATUS) & 1;
+  s = v_readb(MMIOBLTSTATUS) & 1;
 
 #define MMIOSETBACKGROUNDCOLOR(c) \
   *(unsigned char *)(MMIO_POINTER + MMIOBACKGROUNDCOLOR) = c;
