@@ -1031,12 +1031,6 @@ static int et4000_init(int force, int par1, int par2)
     if (new != 0x33)
 	et4000_chiptype = CHIP_ET4000;
     else {
-	/* ET4000/W32. */
-	if (getenv("IOPERM") == NULL && iopl(3) < 0)
-	    /* Can't get further I/O permissions -- */
-	    /* assume ET4000/W32. */
-	    et4000_chiptype = CHIP_ET4000W32;
-	else {
 	    outb(0x217a, 0xec);
 	    val = port_in(0x217b) >> 4;
 	    switch (val) {
@@ -1052,11 +1046,6 @@ static int et4000_init(int force, int par1, int par2)
 	    default:
 		et4000_chiptype = CHIP_ET4000W32p;
 	    }
-	    /* Do NOT set back iopl if set before, since */
-	    /* caller most likely still needs it! */
-	    if (getenv("IOPERM") == NULL)
-		iopl(0);
-	}
     }
 
     if (force)
