@@ -168,7 +168,7 @@ static void neo_getmodeinfo(int mode, vga_modeinfo *modeinfo)
     if (modeinfo->bytesperpixel >= 1) {
 	if(neo_linear_base)modeinfo->flags |= CAPABLE_LINEAR;
         if (__svgalib_neo_inlinearmode())
-	    modeinfo->flags |= IS_LINEAR;
+	    modeinfo->flags |= IS_LINEAR | LINEAR_MODE;
     }
 #endif
 }
@@ -729,20 +729,9 @@ static int neo_test(void)
 {
     unsigned long buf[64];
     int found=0;
-    int _ioperm=0;
-    
-    if (getenv("IOPERM") == NULL) {
-      _ioperm=1;
-      if (iopl(3) < 0) {
-        fprintf(stderr,"svgalib: cannot get I/O permissions\n");
-        exit(1);
-      }
-    }
     
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
     
-    if (_ioperm) iopl(0);
-    
     if (found == 0){ /* found */
       switch ((buf[0] >> 16) & 0xffff){
       case PCI_CHIP_NM2070: case PCI_CHIP_NM2090: case PCI_CHIP_NM2093:
@@ -860,7 +849,6 @@ static int neo_init(int force, int par1, int par2)
 {
     unsigned long buf[64];
     int found=0;
-    int _ioperm=0;
     int w;
 
     vgaIOBase = (port_in(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
@@ -873,15 +861,8 @@ static int neo_init(int force, int par1, int par2)
       NeoChipset = -1;
     };
 
-    if (getenv("IOPERM") == NULL) {
-      _ioperm=1;
-      if (iopl(3) < 0) {
-        fprintf(stderr,"svgalib: cannot get I/O permissions\n");
-        exit(1);
-      }
-    }
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
-    if (_ioperm) iopl(0);
+
     neo_linear_base=0;
     if (!found){ /* found */
       neo_linear_base=buf[4]&0xffffff00;
@@ -969,7 +950,9 @@ static int neo_init(int force, int par1, int par2)
 	NeoPanelWidth  = 640;
 	NeoPanelHeight = 480;
     }
-
+    
+	__svgalib_modeinfo_linearset |= IS_LINEAR;
+	
     cardspecs->flags = INTERLACE_DIVIDE_VERT | CLOCK_PROGRAMMABLE;
     cardspecs->maxHorizontalCrtc = 2040;
     cardspecs->maxPixelClock4bpp = 0;
