@@ -175,7 +175,7 @@ static void oak_getmodeinfo(int mode, vga_modeinfo * modeinfo)
 	modeinfo->aperture_size = oak_memorydetect();
 	modeinfo->set_aperture_page = oak_setpage;
 	if (oak_inlinearmode()) {
-	    modeinfo->flags |= IS_LINEAR;
+	    modeinfo->flags |= IS_LINEAR | LINEAR_MODE;
 	    /* The following line was missing? */
 	    modeinfo->linear_aperture = linearframebuffer;
 	}
@@ -673,13 +673,6 @@ static void oak_setlinearmode(void)
     int temp1;
     temp1 = ((OAKGetByte(OTI87_STATUS) & 0x06) << 1) | 0xe1;
     OAKSetByte(OTI87_VIDMAP, temp1);
-/*    linearframebuffer = (unsigned char *) mmap((caddr_t) 0x40000000,
-					       oak_memory * 1024,
-					       PROT_READ | PROT_WRITE,
-					       MAP_SHARED | MAP_FIXED,
-					       __svgalib_mem_fd,
-					       0xe00000);
-*/
     linearframebuffer=LINEAR_POINTER;
     VGAMemory_Save = __svgalib_graph_mem;
     __svgalib_modeinfo_linearset |= IS_LINEAR;
