@@ -31,30 +31,6 @@ static enum {cPM2=1, cPM2V} chiptype;
 
 static CardSpecs *cardspecs;
 
-static void setpage(int page)
-{
-	static int oldpage = 0;
-
-	if (page != oldpage)
-	{
-		if (page)
-			mmap(BANKED_POINTER,
-			     __svgalib_banked_mem_size,
-			     PROT_READ | PROT_WRITE,
-			     MAP_SHARED | MAP_FIXED,
-			     __svgalib_mem_fd, 
-                             __svgalib_banked_mem_base +  (page << 16));
-		else
-			mmap(BANKED_POINTER,
-			     __svgalib_banked_mem_size,
-			     PROT_READ | PROT_WRITE,
-			     MAP_SHARED | MAP_FIXED,
-			     __svgalib_mem_fd,
-			     __svgalib_banked_mem_base);
-		oldpage = page;
-	}
-}
-
 static int __svgalib_inlinearmode(void)
 {
 return is_linear;
@@ -76,7 +52,7 @@ static void getmodeinfo(int mode, vga_modeinfo *modeinfo)
     if (modeinfo->bytesperpixel >= 1) {
 	if(linear_base)modeinfo->flags |= CAPABLE_LINEAR;
         if (__svgalib_inlinearmode())
-	    modeinfo->flags |= IS_LINEAR;
+	    modeinfo->flags |= IS_LINEAR | LINEAR_MODE;
     }
 }
 
@@ -700,7 +676,7 @@ DriverSpecs __svgalib_pm2_driverspecs =
     lock,
     test,
     init,
-    setpage,
+    __svgalib_emul_setpage,
     NULL,
     NULL,
     setmode,
@@ -754,8 +730,9 @@ static int init(int force, int par1, int par2)
 
     linear_base = buf[6] & 0xffffc000;
     mmio_base = buf[4] & 0xffffc000;
-    MMIO_POINTER = mmap(0, 64*1024, PROT_READ | PROT_WRITE, MAP_SHARED, __svgalib_mem_fd, 
-		    mmio_base);
+    __svgalib_mmio_base=mmio_base;
+    __svgalib_mmio_size=64*1024;
+    map_mmio();
     
     memory = (((GLINT_READ_REG(PMMemConfig) >> 29) & 0x03) + 1) * 2048;
 
@@ -770,10 +747,10 @@ static int init(int force, int par1, int par2)
 	fprintf(stderr,"Using PM2 driver, %iKB.\n",memory);
     };
     
-    munmap(MMIO_POINTER, 64*1024);
-
     pm2_mapio();
-
+    
+	__svgalib_modeinfo_linearset |= IS_LINEAR;
+	
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = memory;
     cardspecs->maxPixelClock4bpp = 75000;	
@@ -781,7 +758,7 @@ static int init(int force, int par1, int par2)
     cardspecs->maxPixelClock16bpp = 230000;	
     cardspecs->maxPixelClock24bpp = 150000;
     cardspecs->maxPixelClock32bpp = 110000;
-    cardspecs->flags = INTERLACE_DIVIDE_VERT | CLOCK_PROGRAMMABLE;
+    cardspecs->flags = INTERLACE_DIVIDE_VERT | CLOCK_PROGRAMMABLE | EMULATE_BANK;
     cardspecs->maxHorizontalCrtc = 4080;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
@@ -793,7 +770,5 @@ static int init(int force, int par1, int par2)
     __svgalib_banked_mem_size=0x10000;
     __svgalib_linear_mem_base=linear_base;
     __svgalib_linear_mem_size=memory*0x400;
-    __svgalib_mmio_base=mmio_base;
-    __svgalib_mmio_size=64*1024;
     return 0;
 }
