@@ -34,79 +34,87 @@ static int r128_ramtype;
 static int BusCntl, CRTOnly, HasPanelRegs;
 
 typedef struct {
-    CARD16        reference_freq;
-    CARD16        reference_div;
-    CARD32        min_pll_freq;
-    CARD32        max_pll_freq;
-    CARD16        xclk;
+    uint16_t        reference_freq;
+    uint16_t        reference_div;
+    uint32_t        min_pll_freq;
+    uint32_t        max_pll_freq;
+    uint16_t        xclk;
 } R128PLLRec, *R128PLLPtr;
 
 typedef struct {
 				/* Common registers */
-    CARD32     ovr_clr;
-    CARD32     ovr_wid_left_right;
-    CARD32     ovr_wid_top_bottom;
-    CARD32     ov0_scale_cntl;
-    CARD32     mpp_tb_config;
-    CARD32     mpp_gp_config;
-    CARD32     subpic_cntl;
-    CARD32     viph_control;
-    CARD32     i2c_cntl_1;
-    CARD32     gen_int_cntl;
-    CARD32     cap0_trig_cntl;
-    CARD32     cap1_trig_cntl;
-    CARD32     bus_cntl;
-    CARD32     config_cntl;
-    CARD32     mem_vga_wp_sel;
-    CARD32     mem_vga_rp_sel;
-    CARD32     surface_cntl;
+    uint32_t     ovr_clr;
+    uint32_t     ovr_wid_left_right;
+    uint32_t     ovr_wid_top_bottom;
+    uint32_t     ov0_scale_cntl;
+    uint32_t     mpp_tb_config;
+    uint32_t     mpp_gp_config;
+    uint32_t     subpic_cntl;
+    uint32_t     viph_control;
+    uint32_t     i2c_cntl_1;
+    uint32_t     gen_int_cntl;
+    uint32_t     cap0_trig_cntl;
+    uint32_t     cap1_trig_cntl;
+    uint32_t     bus_cntl;
+    uint32_t     bus_cntl1;
+    uint32_t     mem_cntl;
+    uint32_t     config_cntl;
+    uint32_t     mem_vga_wp_sel;
+    uint32_t     mem_vga_rp_sel;
+    uint32_t     surface_cntl;
+    uint32_t	 dac_cntl2;
+    uint32_t	 crtc_more_cntl;
+    uint32_t	 dac_ext_cntl;
+    uint32_t	 grph_buf_cntl;
+    uint32_t	 vga_buf_cntl;
+
 				/* Other registers to save for VT switches */
-    CARD32     dp_datatype;
-    CARD32     gen_reset_cntl;
-    CARD32     clock_cntl_index;
-    CARD32     amcgpio_en_reg;
-    CARD32     amcgpio_mask;
+    uint32_t     dp_datatype;
+    uint32_t     gen_reset_cntl;
+    uint32_t     clock_cntl_index;
+    uint32_t     amcgpio_en_reg;
+    uint32_t     amcgpio_mask;
 				/* CRTC registers */
-    CARD32     crtc_gen_cntl;
-    CARD32     crtc_ext_cntl;
-    CARD32     dac_cntl;
-    CARD32     crtc_h_total_disp;
-    CARD32     crtc_h_sync_strt_wid;
-    CARD32     crtc_v_total_disp;
-    CARD32     crtc_v_sync_strt_wid;
-    CARD32     crtc_offset;
-    CARD32     crtc_offset_cntl;
-    CARD32     crtc_pitch;
+    uint32_t     crtc_gen_cntl;
+    uint32_t     crtc_ext_cntl;
+    uint32_t     dac_cntl;
+    uint32_t     crtc_h_total_disp;
+    uint32_t     crtc_h_sync_strt_wid;
+    uint32_t     crtc_v_total_disp;
+    uint32_t     crtc_v_sync_strt_wid;
+    uint32_t     crtc_offset;
+    uint32_t     crtc_offset_cntl;
+    uint32_t     crtc_pitch;
 				/* CRTC2 registers */
-    CARD32     crtc2_gen_cntl;
+    uint32_t     crtc2_gen_cntl;
 				/* Flat panel registers */
-    CARD32     fp_crtc_h_total_disp;
-    CARD32     fp_crtc_v_total_disp;
-    CARD32     fp_gen_cntl;
-    CARD32     fp_h_sync_strt_wid;
-    CARD32     fp_horz_stretch;
-    CARD32     fp_panel_cntl;
-    CARD32     fp_v_sync_strt_wid;
-    CARD32     fp_vert_stretch;
-    CARD32     lvds_gen_cntl;
-    CARD32     tmds_crc;
+    uint32_t     fp_crtc_h_total_disp;
+    uint32_t     fp_crtc_v_total_disp;
+    uint32_t     fp_gen_cntl;
+    uint32_t     fp_h_sync_strt_wid;
+    uint32_t     fp_horz_stretch;
+    uint32_t     fp_panel_cntl;
+    uint32_t     fp_v_sync_strt_wid;
+    uint32_t     fp_vert_stretch;
+    uint32_t     lvds_gen_cntl;
+    uint32_t     tmds_crc;
 				/* Computed values for PLL */
-    CARD32     dot_clock_freq;
-    CARD32     pll_output_freq;
+    uint32_t     dot_clock_freq;
+    uint32_t     pll_output_freq;
     int        feedback_div;
     int        post_div;
 				/* PLL registers */
-    CARD32     ppll_ref_div;
-    CARD32     ppll_div_3;
-    CARD32     htotal_cntl;
+    uint32_t     ppll_ref_div;
+    uint32_t     ppll_div_3;
+    uint32_t     htotal_cntl;
 				/* DDA register */
-    CARD32     dda_config;
-    CARD32     dda_on_off;
-    CARD32     vga_dda_config;
-    CARD32     vga_dda_on_off;
+    uint32_t     dda_config;
+    uint32_t     dda_on_off;
+    uint32_t     vga_dda_config;
+    uint32_t     vga_dda_on_off;
 				/* Pallet */
     Bool       palette_valid;
-    CARD32     palette[256];
+    uint32_t     palette[256];
 } R128SaveRec, *R128SavePtr;
 
 typedef struct {        /* All values in XCLKS    */
@@ -190,10 +198,19 @@ static void R128RestoreCommonRegisters(R128SavePtr restore)
     OUTREG(R128_CAP0_TRIG_CNTL,       restore->cap0_trig_cntl);
     OUTREG(R128_CAP1_TRIG_CNTL,       restore->cap1_trig_cntl);
     OUTREG(R128_BUS_CNTL,             restore->bus_cntl);
+    OUTREG(R128_BUS_CNTL1,            restore->bus_cntl1);
     OUTREG(R128_CONFIG_CNTL,          restore->config_cntl);
     OUTREG(R128_MEM_VGA_WP_SEL,	      restore->mem_vga_wp_sel);
     OUTREG(R128_MEM_VGA_RP_SEL,	      restore->mem_vga_rp_sel);
-    OUTREG(RADEON_SURFACE_CNTL,	      restore->surface_cntl);
+
+    if(chiptype == Rage128) {
+        OUTREG(RADEON_SURFACE_CNTL,   restore->surface_cntl);
+        OUTREG(RADEON_DAC_CNTL2,     restore->dac_cntl2);
+        OUTREG(RADEON_CRTC_MORE_CNTL,restore->crtc_more_cntl);
+        OUTREG(RADEON_DAC_EXT_CNTL,  restore->dac_ext_cntl);
+        OUTREG(RADEON_GRPH_BUF_CNTL, restore->grph_buf_cntl);
+        OUTREG(RADEON_VGA_BUF_CNTL,  restore->vga_buf_cntl);
+    }
 
 }
 
@@ -221,7 +238,7 @@ static void R128RestoreCrtcRegisters(R128SavePtr restore)
 #if 0
 static void R128RestoreFPRegisters(R128SavePtr restore)
 {
-    CARD32        tmp;
+    uint32_t        tmp;
 
     OUTREG(R128_CRTC2_GEN_CNTL,       restore->crtc2_gen_cntl);
     OUTREG(R128_FP_CRTC_H_TOTAL_DISP, restore->fp_crtc_h_total_disp);
@@ -362,7 +379,7 @@ if (!restore->palette_valid) return;
 //    if (info->HasPanelRegs) PAL_SELECT(0);
 
     OUTPAL_START(0);
-    for (i = 0; i < 256; i++) OUTPAL_NEXT_CARD32(restore->palette[i]);
+    for (i = 0; i < 256; i++) OUTPAL_NEXT_uint32_t(restore->palette[i]);
 }
 
 /* Write out state to define a new video mode.  */
@@ -395,7 +412,7 @@ static void R128RestoreMode(R128SavePtr restore)
     if(chiptype == Rage128) {
 	R128RestoreDDARegisters(restore);
     }
-    	R128RestorePalette(restore);
+    R128RestorePalette(restore);
 }
 
 /* Read common registers. */
@@ -414,11 +431,20 @@ static void R128SaveCommonRegisters(R128SavePtr save)
     save->cap0_trig_cntl     = INREG(R128_CAP0_TRIG_CNTL);
     save->cap1_trig_cntl     = INREG(R128_CAP1_TRIG_CNTL);
     save->bus_cntl           = INREG(R128_BUS_CNTL);
+    save->bus_cntl1          = INREG(R128_BUS_CNTL1);
+    save->mem_cntl           = INREG(R128_MEM_CNTL);
     save->config_cntl        = INREG(R128_CONFIG_CNTL);
     save->mem_vga_wp_sel     = INREG(R128_MEM_VGA_WP_SEL);
     save->mem_vga_rp_sel     = INREG(R128_MEM_VGA_RP_SEL);
-	if(chiptype==Radeon)
+
+    if(chiptype==Radeon) {
     	save->surface_cntl   = INREG(RADEON_SURFACE_CNTL);
+        save->dac_cntl2	     = INREG(RADEON_DAC_CNTL2);
+        save->crtc_more_cntl = INREG(RADEON_CRTC_MORE_CNTL);
+        save->dac_ext_cntl   = INREG(RADEON_DAC_EXT_CNTL);
+        save->grph_buf_cntl  = INREG(RADEON_GRPH_BUF_CNTL);
+        save->vga_buf_cntl  =  INREG(RADEON_VGA_BUF_CNTL);
+    }
 }
 
 /* Read CRTC registers. */
@@ -521,6 +547,7 @@ static void R128InitCommonRegisters(R128SavePtr save)
     save->mem_vga_rp_sel     = 0;
     save->config_cntl        = INREG(R128_CONFIG_CNTL);
     save->bus_cntl           = BusCntl;
+    save->bus_cntl1          = INREG(R128_BUS_CNTL1);
     if(chiptype == Radeon) {
         if(save->bus_cntl & RADEON_BUS_READ_BURST) 
             save->bus_cntl |=RADEON_BUS_RD_DISCARD_EN;
@@ -633,7 +660,14 @@ static Bool R128InitCrtcRegisters(R128SavePtr save,
     
     if(chiptype == Radeon) {
         save->crtc_pitch |= save->crtc_pitch<<16;
-        save->surface_cntl = RADEON_SURF_TRANSLATION_DIS;        
+        save->surface_cntl = RADEON_SURF_TRANSLATION_DIS;
+
+        save->dac_cntl2	     = INREG(RADEON_DAC_CNTL2);
+        save->crtc_more_cntl = INREG(RADEON_CRTC_MORE_CNTL);
+        save->dac_ext_cntl   = INREG(RADEON_DAC_EXT_CNTL);
+        save->grph_buf_cntl  = INREG(RADEON_GRPH_BUF_CNTL);
+        save->vga_buf_cntl   =  INREG(RADEON_VGA_BUF_CNTL);
+
     }
 
 #ifdef __PPC
@@ -933,30 +967,6 @@ static int r128_is_linear, r128_linear_base, r128_mmio_base;
 
 static CardSpecs *cardspecs;
 
-static void radeon_setpage(int page)
-{
-	static int oldpage = 0;
-
-	if (page != oldpage)
-	{
-		if (page)
-			mmap(BANKED_POINTER,
-			     __svgalib_banked_mem_size,
-			     PROT_READ | PROT_WRITE,
-			     MAP_SHARED | MAP_FIXED,
-			     __svgalib_mem_fd, 
-                             __svgalib_banked_mem_base +  (page << 16));
-		else
-			mmap(BANKED_POINTER,
-			     __svgalib_banked_mem_size,
-			     PROT_READ | PROT_WRITE,
-			     MAP_SHARED | MAP_FIXED,
-			     __svgalib_mem_fd,
-			     __svgalib_banked_mem_base);
-		oldpage = page;
-	}
-}
-
 static void r128_setpage(int page)
 {
     page*=2;
@@ -987,7 +997,7 @@ static void r128_getmodeinfo(int mode, vga_modeinfo *modeinfo)
     if (modeinfo->bytesperpixel >= 1) {
 	if(r128_linear_base)modeinfo->flags |= CAPABLE_LINEAR;
         if (__svgalib_r128_inlinearmode())
-	    modeinfo->flags |= IS_LINEAR;
+	    modeinfo->flags |= IS_LINEAR | LINEAR_MODE;
     }
 }
 
@@ -1369,8 +1379,8 @@ static int r128_init(int force, int par1, int par2)
 #define R128_BIOS16(x) (*(unsigned short *)(BIOS_POINTER + x))
 #define R128_BIOS32(x) (*(unsigned int *)(BIOS_POINTER + x))
     {
-        CARD16 bios_header;
-        CARD16 pll_info_block;
+        uint16_t bios_header;
+        uint16_t pll_info_block;
         BIOS_POINTER = mmap(0, 64*1024, PROT_READ | PROT_WRITE, MAP_SHARED, __svgalib_mem_fd,
        			    0xc0000);
         bios_header    = R128_BIOS16(0x48);
@@ -1394,7 +1404,9 @@ fprintf(stderr,"pll: %i %i %i %i %i\n",pll.reference_freq,pll.reference_div,
     if (__svgalib_driver_report) {
 	fprintf(stderr,"Using ATI %s driver, %iKB.\n",chipnames[chiptype],r128_memory);
     };
-
+    
+	__svgalib_modeinfo_linearset |= IS_LINEAR;
+	
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = r128_memory;
     cardspecs->maxPixelClock4bpp = 75000;	
@@ -1418,7 +1430,7 @@ fprintf(stderr,"pll: %i %i %i %i %i\n",pll.reference_freq,pll.reference_div,
     __svgalib_mmio_size=16*1024;
     if(chiptype==Radeon) {
         __svgalib_banked_mem_base=r128_linear_base;
-        __svgalib_r128_driverspecs.__svgalib_setpage = radeon_setpage;
+        __svgalib_r128_driverspecs.__svgalib_setpage = __svgalib_emul_setpage;
     }
 
     return 1;
