@@ -50,13 +50,13 @@
 
 #define R128_TIMEOUT 2000000
 
-#define MMIO_IN8(dummy, addr)  (*(unsigned char *)(MMIO_POINTER + addr))
-#define MMIO_IN16(dummy, addr)  (*(unsigned short *)(MMIO_POINTER + addr))
-#define MMIO_IN32(dummy, addr)  LE32(*(unsigned int *)(MMIO_POINTER + addr))
+#define MMIO_IN8(dummy, addr)  v_readb(addr)
+#define MMIO_IN16(dummy, addr)  v_readw(addr)
+#define MMIO_IN32(dummy, addr)  LE32(v_readl(addr))
 
-#define MMIO_OUT8(dummy, addr, val)  (*(unsigned char *)(MMIO_POINTER + addr) = val)
-#define MMIO_OUT16(dummy, addr, val)  (*(unsigned short *)(MMIO_POINTER + addr) = val)
-#define MMIO_OUT32(dummy, addr, val)  (*(unsigned int *)(MMIO_POINTER + addr) = LE32(val))
+#define MMIO_OUT8(dummy, addr, val)  v_writeb(val, addr)
+#define MMIO_OUT16(dummy, addr, val) v_writew(val, addr)
+#define MMIO_OUT32(dummy, addr, val) v_writel(LE32(val), addr)
 
 				/* Memory mapped register access macros */
 #define INREG8(addr)        MMIO_IN8(R128MMIO, addr)
@@ -66,8 +66,6 @@
 #define OUTREG16(addr, val) MMIO_OUT16(R128MMIO, addr, val)
 #define OUTREG(addr, val)   MMIO_OUT32(R128MMIO, addr, val)
 
-#define ADDRREG(addr)       ((volatile CARD32 *)(R128MMIO + (addr)))
-
 #if 0
 #define R128_BIOS8(v)  (VBIOS[v])
 #define R128_BIOS16(v) (VBIOS[v] | \
@@ -80,7 +78,7 @@
                         
 #define OUTREGP(addr, val, mask)   \
     do {                           \
-	CARD32 tmp = INREG(addr);  \
+	uint32_t tmp = INREG(addr);  \
 	tmp &= (mask);             \
 	tmp |= (val);              \
 	OUTREG(addr, tmp);         \
@@ -96,7 +94,7 @@
 
 #define OUTPLLP(addr, val, mask)                                   \
     do {                                                                  \
-	CARD32 tmp = INPLL(addr);                                  \
+	uint32_t tmp = INPLL(addr);                                  \
 	tmp &= (mask);                                                    \
 	tmp |= (val);                                                     \
 	OUTPLL(addr, tmp);                                                \
@@ -112,7 +110,7 @@
 	OUTREG(R128_PALETTE_DATA, ((r) << 16) | ((g) << 8) | (b));        \
     } while (0)
 
-#define OUTPAL_NEXT_CARD32(v)                                             \
+#define OUTPAL_NEXT_uint32_t(v)                                             \
     do {                                                                  \
 	OUTREG(R128_PALETTE_DATA, (v & 0x00ffffff));                      \
     } while (0)
@@ -1487,6 +1485,12 @@
 #define R128_CCE_VC_CNTL_PRIM_WALK_RING          0x00000030
 #define R128_CCE_VC_CNTL_NUM_SHIFT               16
 
+#define RADEON_DAC_CNTL2			 0x007c
+#define RADEON_CRTC_MORE_CNTL			 0x027c
+#define RADEON_DAC_EXT_CNTL			 0x0280
+#define RADEON_GRPH_BUF_CNTL			 0x02f0
+#define RADEON_VGA_BUF_CNTL			 0x02f4
+
 #define RADEON_SURFACE_CNTL			 0x0b00
 
 #define RADEON_VCLK_SRC_SEL_MASK		 0x0003
