@@ -379,7 +379,7 @@ static void rage_getmodeinfo(int mode, vga_modeinfo *modeinfo)
     if (modeinfo->bytesperpixel >= 1) {
 	if(rage_linear_base)modeinfo->flags |= CAPABLE_LINEAR;
         if (__svgalib_rage_inlinearmode())
-	    modeinfo->flags |= IS_LINEAR;
+	    modeinfo->flags |= IS_LINEAR | LINEAR_MODE;
     }
 }
 
@@ -1241,6 +1241,7 @@ static int rage_init(int force, int par1, int par2)
          cardspecs->maxPixelClock24bpp = 60000;
          cardspecs->maxPixelClock32bpp = 60000;
    };
+   __svgalib_modeinfo_linearset |= IS_LINEAR;
    cardspecs->flags = CLOCK_PROGRAMMABLE;
    cardspecs->maxHorizontalCrtc = 4088;
    cardspecs->nClocks = 0;
