@@ -28,7 +28,8 @@ And XFree 4.1.0 driver by Kevin Brosius.
 
 typedef struct {     
 
-   unsigned char SR08, SR0A, SR0F, SR10, SR11, SR12, SR13, SR15, SR18, SR29; /* SR9-SR1C, ext seq. */
+   unsigned char SR08, SR0A, SR0E, SR0F, SR10, SR11, SR12, SR13;
+   unsigned char SR15, SR18, SR1B, SR29, SR30;
    unsigned char SR54, SR55, SR56, SR57;
    unsigned char Clock;
 //   unsigned char s3DacRegs[0x101];
@@ -80,7 +81,7 @@ static void getmodeinfo(int mode, vga_modeinfo *modeinfo)
     if (modeinfo->bytesperpixel >= 1) {
 	if(linear_base)modeinfo->flags |= CAPABLE_LINEAR;
         if (inlinearmode())
-	    modeinfo->flags |= IS_LINEAR;
+	    modeinfo->flags |= IS_LINEAR | LINEAR_MODE;
     }
 }
 
@@ -195,6 +196,7 @@ static int saveregs(unsigned char regs[])
         save->CR6D = __svgalib_incrtc(0x6d);
 
    /* Save sequencer extended regs for DCLK PLL programming */
+    save->SR0E = __svgalib_inseq(0x0E);
     save->SR10 = __svgalib_inseq(0x10);
     save->SR11 = __svgalib_inseq(0x11);
     save->SR12 = __svgalib_inseq(0x12);
@@ -203,7 +205,6 @@ static int saveregs(unsigned char regs[])
     if(chipset>=SAVAGE3D) 
         save->SR29 = __svgalib_inseq(0x29);
 
-
     if((chipset==TRIO3D2X)||(chipset==VIRGEGX2)||(chipset==VIRGEMX)) {
         save->SR29 = __svgalib_inseq(0x29);
         save->SR54 = __svgalib_inseq(0x54);
@@ -213,7 +214,14 @@ static int saveregs(unsigned char regs[])
     }
         
     save->SR15 = __svgalib_inseq(0x15);
-    save->SR18 = __svgalib_inseq(0x18);
+
+	if(chipset>=SAVAGE3D) 
+    	save->SR30 = __svgalib_inseq(0x30);
+
+	save->SR18 = __svgalib_inseq(0x18);
+
+	if(chipset>=SAVAGE3D) 
+    	save->SR1B = __svgalib_inseq(0x1B);
 
     if(chipset<=TRIO3D) {
         save->SR0A = __svgalib_inseq(0x0a);
@@ -370,16 +378,23 @@ static void setregs(const unsigned char regs[], int mode)
 
     __svgalib_outseq(0x18, restore->SR18);
 
+    if(chipset>=SAVAGE3D)
+        __svgalib_outseq(0x1B, restore->SR1B);
+
     tmp = __svgalib_inseq(0x15) & ~0x21;
     __svgalib_outseq(0x15, tmp | 0x03);
     __svgalib_outseq(0x15, tmp | 0x23);
     __svgalib_outseq(0x15, tmp | 0x03);
     __svgalib_outseq(0x15, restore->SR15);
     
+	usleep(100);
+	
     if(chipset<=TRIO3D) {
         __svgalib_outseq(0x0a, restore->SR0A);
         __svgalib_outseq(0x0f, restore->SR0F);
-    }
+    } else if (chipset >= SAVAGE3D) {
+		__svgalib_outseq(0x30, restore->SR30);
+	}
 
     __svgalib_outseq(0x08, restore->SR08);
 
@@ -590,6 +605,9 @@ static void initializemode(unsigned char *moderegs,
     new->SR10 = 255; /* This is a reserved value, so we use as flag */
     new->SR11 = 255;
 
+	new->SR1B = 0;
+	new->SR30 = __svgalib_inseq(0x30);
+
     switch( modeinfo->colorBits ) {
         case 8:
             new->CR67 = 0x00;   /* 8bpp, 1 pixel/clock */
@@ -875,6 +893,8 @@ static int test(void)
         case 0x8c10:
         case 0x8c12:
         case 0x9102:
+		case 0x8d03:
+		case 0x8d04:
             init(0,0,0);
             return 1;
             break;
@@ -1191,6 +1211,9 @@ static int init(int force, int par1, int par2)
             break;
         case 0x8a22:
         case 0x8a23:
+
+		case 0x8d03:
+		case 0x8d04:
             chipset = SAVAGE4;
             break;
         case 0x9102:
@@ -1206,8 +1229,10 @@ static int init(int force, int par1, int par2)
 #if 1 /* You need to write linear address to CR59 5A, and enable MMIO in CR53 - 
          But how to do it if it's a secondary card??? */
         if(__svgalib_secondary) {
-            __svgalib_vgammbase=mmap(0,0x1000,PROT_READ|PROT_WRITE,MAP_SHARED,
-                		     __svgalib_mem_fd,mmio_base+0x8000); 
+            __svgalib_mmio_base = mmio_base;
+            __svgalib_mmio_size = 0x10000;
+            map_mmio();
+            __svgalib_vgammbase=0x8000; 
             __svgalib_mm_io_mapio();
         }
 #endif
@@ -1217,8 +1242,11 @@ static int init(int force, int par1, int par2)
     } else {
         linear_base=buf[5]&0xffffff00;
         mmio_base  =buf[4]&0xffffff00;
+        __svgalib_mmio_base = mmio_base;
+        __svgalib_mmio_size = 0x10000;
+        map_mmio();
     
-        __svgalib_vgammbase=mmap(0,0x1000,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,mmio_base+0x8000);
+        __svgalib_vgammbase=0x8000;
         __svgalib_mm_io_mapio();
         unlock();
     
@@ -1244,7 +1272,6 @@ static int init(int force, int par1, int par2)
     if (__svgalib_driver_report) {
 	fprintf(stderr,"Using SAVAGE driver, %iKB. Chipset: %s\n",memory, chipnames[chipset]);
     };
-
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = memory;
     cardspecs->maxPixelClock4bpp = 0;	
@@ -1263,7 +1290,5 @@ static int init(int force, int par1, int par2)
     __svgalib_banked_mem_size=0x10000;
     __svgalib_linear_mem_base=linear_base;
     __svgalib_linear_mem_size=memory*0x400;
-    __svgalib_mmio_base = mmio_base;
-    __svgalib_mmio_size = 0x10000;
     return 0;
 }
