@@ -740,14 +740,6 @@ bit 0:		force noninterlaced if set
 	}
     }
 
-    /* The 9440 uses ports 43C8 and 43C9... we need more privileges */
-    if (tvga_model == 9440 || tvga_model == 9680)
-	if (getenv("IOPERM") == NULL)
-            if (iopl(3) < 0) {
-		fprintf(stderr,"tvga%d: Cannot get I/O permissions\n",tvga_model);
-    }
-
-
     if (__svgalib_driver_report) {
 	if(tvga_model == 9440)
 	    fprintf(stderr,"Using Trident 9440 driver (%dK)\n",
