@@ -85,7 +85,7 @@ static void vesa_getmodeinfo(int mode, vga_modeinfo *modeinfo)
       if (modeinfo->bytesperpixel >= 1) {
 	modeinfo->flags |= CAPABLE_LINEAR;
         if (__svgalib_vesa_inlinearmode())
-	    modeinfo->flags |= IS_LINEAR;
+	    modeinfo->flags |= IS_LINEAR | LINEAR_MODE;
       }
       
     /* to get the logical scanline width */
@@ -104,7 +104,7 @@ static void vesa_getmodeinfo(int mode, vga_modeinfo *modeinfo)
 }
 
 /* Read and save chipset-specific registers */
-static int vesa_saveregs(unsigned char regs[])
+static int vesa_saveregs(uint8_t regs[])
 { 
   void * buf;
   buf=LRMI_mem1;
@@ -120,7 +120,7 @@ static int vesa_saveregs(unsigned char regs[])
 
 /* Set chipset-specific registers */
 
-static void vesa_setregs(const unsigned char regs[], int mode)
+static void vesa_setregs(const uint8_t regs[], int mode)
 {   
 
   void * buf;
@@ -356,7 +356,7 @@ static int vesa_init(int force, int par1, int par2)
     short int *mode_list;
     int i;
 #if 0
-    unsigned char *m;
+    uint8_t *m;
     int address;
 
     m=mmap(0,0x502,PROT_READ,MAP_SHARED,__svgalib_mem_fd,0);
