@@ -9,13 +9,11 @@
 #define _LIBVGA_H
 
 #include <string.h>
-# if defined __GLIBC__ && __GLIBC__ >= 2
-#include <sys/io.h>
-#endif
 
-typedef unsigned int CARD32;
-typedef unsigned short CARD16;
-typedef unsigned char CARD8;
+#include <stdint.h>
+//typedef unsigned int CARD32;
+//typedef unsigned short CARD16;
+//typedef unsigned char CARD8;
 
 /* --------------------- Macro definitions shared by library modules */
 
@@ -122,12 +120,13 @@ struct info {
 extern int __svgalib_CRT_I;		/* current CRT index register address */
 extern int __svgalib_CRT_D;		/* current CRT data register address */
 extern int __svgalib_IS1_R;		/* current input status register address */
-extern unsigned char * BANKED_POINTER, * LINEAR_POINTER;
-extern unsigned char * MMIO_POINTER;
-extern unsigned long int __svgalib_banked_mem_base, __svgalib_banked_mem_size;
-extern unsigned long int __svgalib_mmio_base, __svgalib_mmio_size;
-extern unsigned long int __svgalib_linear_mem_base, __svgalib_linear_mem_size;
-extern unsigned long int __svgalib_mmio_base, __svgalib_mmio_size;
+extern uint8_t * BANKED_POINTER, * LINEAR_POINTER;
+extern uint8_t *MMIO_POINTER;
+extern uint8_t *SPARSE_MMIO;
+extern unsigned long __svgalib_banked_mem_base, __svgalib_banked_mem_size;
+extern unsigned long __svgalib_mmio_base, __svgalib_mmio_size;
+extern unsigned long __svgalib_linear_mem_base, __svgalib_linear_mem_size;
+extern unsigned long __svgalib_mmio_base, __svgalib_mmio_size;
 extern struct info CI;		/* current video parameters */
 extern int COL;			/* current color            */
 extern int CM;			/* current video mode       */
@@ -184,10 +183,8 @@ extern int __svgalib_getchipset(void);
 extern int __svgalib_name2number(char *modename);
 extern void __svgalib_delay(void);
 extern int __svgalib_addmode(int xdim, int ydim, int cols, int xbytes, int bytespp);
-extern int __svgalib_physmem(void);
 extern void __svgalib_waitvtactive(void);
 extern void __svgalib_open_devconsole(void);
-extern void __svgalib_flipaway(void);
 extern void (*__svgalib_mouse_eventhandler) (int, int, int, int, int, int, int);
 extern void (*__svgalib_keyboard_eventhandler) (int, int);
 extern void __joystick_flip_vc(int acquire);
@@ -200,6 +197,11 @@ extern int __svgalib_mapkeyname(const char *keyname);
 extern void __svgalib_mouse_update_keymap(void);
 extern int __svgalib_vgacolor(void);
 extern void __svgalib_cursor_restore(void);
+extern void map_mmio(void);
+extern void map_mem(void);
+extern void map_linear(unsigned long, unsigned long);
+extern void unmap_linear(unsigned long);
+extern void __svgalib_emul_setpage(int);
 
 #if 0 
 /* remove this part ? */
@@ -207,12 +209,12 @@ extern void __svgalib_releasevt_signal(int n);
 extern void __svgalib_acquirevt_signal(int n);
 #endif
 
-#define gr_readb(off)		(((volatile unsigned char *)GM)[(off)])
-#define gr_readw(off)		(*(volatile unsigned short*)((GM)+(off)))
-#define gr_readl(off)		(*(volatile unsigned long*)((GM)+(off)))
+#define gr_readb(off)		(((volatile uint8_t *)GM)[(off)])
+#define gr_readw(off)		(*(volatile uint16_t*)((GM)+(off)))
+#define gr_readl(off)		(*(volatile uint32_t*)((GM)+(off)))
 #define gr_writeb(v,off)	(GM[(off)] = (v))
-#define gr_writew(v,off)	(*(unsigned short*)((GM)+(off)) = (v))
-#define gr_writel(v,off)	(*(unsigned long*)((GM)+(off)) = (v))
+#define gr_writew(v,off)	(*(uint16_t*)((GM)+(off)) = (v))
+#define gr_writel(v,off)	(*(uint32_t*)((GM)+(off)) = (v))
 
 extern void port_out(int value, int port);
 extern void port_outw(int value, int port);
@@ -267,6 +269,11 @@ extern unsigned char *__svgalib_give_graph_blue(void);
 #define DTP(x)
 #endif
 
+#ifdef DEBUG
+#define DPRINTF(args...) fprintf(stderr, args)
+#else
+#define DPRINTF(...)
+#endif
 
 #endif /* _LIBVGA_H */
 
