@@ -115,7 +115,7 @@ static void GENDAC_SDAC_init(void)
     /* Calculate MCLK in kHz. */
     MCLK = 14318 * (m + 2) / (n1 + 2) / (1 << n2);
     if (__svgalib_driver_report)
-	printf("svgalib: S3-GENDAC/SDAC: MCLK = %d.%03d MHz\n",
+	fprintf(stderr,"svgalib: S3-GENDAC/SDAC: MCLK = %d.%03d MHz\n",
 	       MCLK / 1000, MCLK % 1000);
 }
 #endif
@@ -145,7 +145,7 @@ static int S3dacsFindClock(int freq_in, int min_n2, int freq_min, int freq_max,
     unsigned char best_n1 = 16 + 2, best_n2 = 2, best_m = 125 + 2;
 
 #if DEBUG_FINDCLOCK
-    printf("S3dacsFindClock: Trying to match clock of %0.3f MHz\n", freq_in / 1000.0);
+    fprintf(stderr,"S3dacsFindClock: Trying to match clock of %0.3f MHz\n", freq_in / 1000.0);
 #endif
     
     ffreq_in = freq_in / 1000.0 / BASE_FREQ;
@@ -155,14 +155,14 @@ static int S3dacsFindClock(int freq_in, int min_n2, int freq_min, int freq_max,
     /* Check if getting freq_in is possible at all */
     if (freq_in < freq_min / 8) {
 #if DEBUG_FINDCLOCK
-	printf("S3dacsFindClock: %0.3f MHz is too low (lowest is %0.3f MHz)\n",
+	fprintf(stderr,"S3dacsFindClock: %0.3f MHz is too low (lowest is %0.3f MHz)\n",
 	       freq_in / 1000.0, freq_min / 1000.0 / 8);
 #endif
 	return 0;
     }  
     if (freq_in > freq_max / (1 << min_n2)) {
 #if DEBUG_FINDCLOCK
-	printf("S3dacsFindClock: %0.3f MHz is too high (highest is %0.3f MHz)\n",
+	fprintf(stderr,"S3dacsFindClock: %0.3f MHz is too high (highest is %0.3f MHz)\n",
 	       freq_in / 1000.0, freq_max / 1000.0 / (1 << min_n2));
 #endif
 	return 0;
@@ -191,7 +191,7 @@ static int S3dacsFindClock(int freq_in, int min_n2, int freq_min, int freq_max,
     }
 
 #if DEBUG_FINDCLOCK
-    printf("S3dacsFindClock: clock wanted %1.6f MHz, found %1.6f MHz (m %d, n1 %d, n2 %d)\n",
+    fprintf(stderr,"S3dacsFindClock: clock wanted %1.6f MHz, found %1.6f MHz (m %d, n1 %d, n2 %d)\n",
 	   freq_in / 1000.0,
 	   best_m / ((double) best_n1 * (1 << best_n2)) * BASE_FREQ,
 	   best_m, best_n1, best_n2);
@@ -248,7 +248,7 @@ static void GENDAC_SDAC_initialize_clock_state(unsigned char *regs, int freq)
     int n, m;
 
     if (!S3dacsFindClock(freq, 0, 100000, 250000, &min_m, &min_n1, &n2)) {
-	printf("Bad dot clock %0.3f MHz.\n", freq / 1000.0);
+	fprintf(stderr,"Bad dot clock %0.3f MHz.\n", freq / 1000.0);
 	return;
     }
     
@@ -257,7 +257,7 @@ static void GENDAC_SDAC_initialize_clock_state(unsigned char *regs, int freq)
     regs[SDAC_PLL_M] = m;
     regs[SDAC_PLL_N1_N2] = n;
     if (__svgalib_driver_report)
-	printf("Initializing DAC PLL values; 0x%02X, 0x%02X.\n", m, n);
+	fprintf(stderr,"Initializing DAC PLL values; 0x%02X, 0x%02X.\n", m, n);
 }
 
 static void GENDAC_SDAC_savestate(unsigned char *regs)
@@ -295,7 +295,7 @@ static void GENDAC_SDAC_restorestate(const unsigned char *regs)
 	n2 = (regs[SDAC_PLL_N1_N2] & 0x60) >> 5;
 
 	clk = 14318 * (m + 2) / (n1 + 2) / (1 << n2);
-	printf("SDAC.restorestate, setting clock 0x%02X 0x%02X (%d.%3dMHz)\n",
+	fprintf(stderr,"SDAC.restorestate, setting clock 0x%02X 0x%02X (%d.%3dMHz)\n",
 	       regs[SDAC_PLL_M],
 	       regs[SDAC_PLL_N1_N2], clk / 1000, clk % 1000);
     } while (0);
@@ -551,11 +551,11 @@ static void Trio64_set_mclk(int khz)
     int min_m, min_n1, n2;
     
     if (!S3dacsFindClock(khz, 0, 40000, 70000, &min_m, &min_n1, &n2)) {
-	printf("Bad MCLK %0.3f MHz.\n", khz / 1000.0);
+	fprintf(stderr,"Bad MCLK %0.3f MHz.\n", khz / 1000.0);
 	return;
     }
 
-    printf("%0.3f MHz MCLK, m = %d, n = %d, r = %d\n", khz / 1000.0, min_m - 2, min_n1 - 2, n2);
+    fprintf(stderr,"%0.3f MHz MCLK, m = %d, n = %d, r = %d\n", khz / 1000.0, min_m - 2, min_n1 - 2, n2);
     outb(0x3C4, 0x08);
     sr8 = inb(0x3C5);
     outb(0x3C5, 0x06);		/* Unlock. */
@@ -581,7 +581,7 @@ static void Trio64_init(void)
 
     mclk = Trio64_get_mclk();
     if (__svgalib_driver_report)
-	printf("svgalib: RAMDAC: Trio64: MCLK = %0.3f MHz\n",
+	fprintf(stderr,"svgalib: RAMDAC: Trio64: MCLK = %0.3f MHz\n",
 	       mclk / 1000.0);
     __svgalib_s3_s3Mclk = mclk;
 }
@@ -609,7 +609,7 @@ static void Trio64_initialize_clock_state(unsigned char *regs, int freq)
     int n, m;
     
     if (!S3dacsFindClock(freq, 0, 130000, 270000, &min_m, &min_n1, &n2)) {
-	printf("Bad dot clock %0.3f MHz.\n", freq / 1000.0);
+	fprintf(stderr,"Bad dot clock %0.3f MHz.\n", freq / 1000.0);
 	return;
     }
     
@@ -722,7 +722,7 @@ static void Trio64_qualify_cardspecs(CardSpecs * cardspecs, int dacspeed)
 {
     if (dacspeed) {
 	if (__svgalib_driver_report)
-	    printf("svgalib: using 'dacspeed' not recommended for this RAMDAC.\n");
+	    fprintf(stderr,"svgalib: using 'dacspeed' not recommended for this RAMDAC.\n");
 	cardspecs->maxPixelClock4bpp = dacspeed;
 	cardspecs->maxPixelClock8bpp = 135000;
 	cardspecs->maxPixelClock16bpp = dacspeed;
