@@ -87,7 +87,7 @@ void __svgalib_waitvtactive(void)
 	    perror("ioctl(VT_WAITACTIVE)");
 	    exit(1);
 	}
-	usleep(150000);
+//	usleep(150000);
     }
 }
 
@@ -115,7 +115,9 @@ void __svgalib_open_devconsole(void)
     struct stat sbuf;
     char fname[30];
 
-    if(__svgalib_novccontrol)return;
+    if(__svgalib_novccontrol) {
+        return;
+    }
 
     if (__svgalib_tty_fd >= 0)
         return;
@@ -160,8 +162,10 @@ void __svgalib_open_devconsole(void)
     /* We must use RDWR to allow for output... */
     if (((__svgalib_tty_fd = open(fname, O_RDWR)) >= 0) &&
         (ioctl(__svgalib_tty_fd, VT_GETSTATE, &vts) >= 0)) {
-        if (!check_owner(vts.v_active))
+        if (!check_owner(vts.v_active)) {
+            
             goto error;
+        }
         /* success, redirect all stdios */
         if (DREP)
             fprintf(stderr,"[svgalib: allocated virtual console #%d]\n", __svgalib_vc);
