@@ -2,6 +2,17 @@
 #include "svgalib_helper.h"
 #include "libvga.h"
 
+void port_rep_outb(unsigned char* string, int length, int port)
+{
+  io_string_t iostr;
+
+  iostr.port = port;
+  iostr.string = string;
+  iostr.length = length;
+  
+  ioctl(__svgalib_mem_fd,SVGALIB_HELPER_IOCSREPOUTB,&iostr); 
+}
+
 void port_out(int value, int port)
 {
     io_t iov;
