@@ -11,7 +11,7 @@
 #include <string.h>
 #include "vga.h"
 #include "libvga.h"
-#include "vgabg.h"
+#include "driver.h"
 #include "ppcmemset.h"
 
 int vga_clear(void)
@@ -47,11 +47,15 @@ int vga_clear(void)
 	    {
 		int i;
 		int pages = (CI.ydim * CI.xbytes + 65535) >> 16;
-		for (i = 0; i < pages; ++i) {
-		    vga_setpage(i);
-		    /* clear video memory */
-                    memset(GM, 0, 65536);
-		}
+                if (__svgalib_modeinfo_linearset & IS_LINEAR ) {
+                    memset(LINEAR_POINTER, 0, pages<<16);
+                } else {
+                    for (i = 0; i < pages; ++i) {
+                        vga_setpage(i);
+		        /* clear video memory */
+                        memset(GM, 0, 65536);
+                    }
+                }
 	    }
 	    break;
 	}
