@@ -42,7 +42,7 @@ void vga_copytoplanar256(unsigned char *virtual, int pitch, int voffset,
 	outb(SEQ_D, 1 << plane);
 	virtualp = virtual;
 #ifdef USE_ASM
-	voffsetp = __svgalib_graph_mem + voffset;
+	voffsetp = GM + voffset;
 #else
 	voff = voffset;
 #endif
