@@ -119,7 +119,6 @@ void MEMCPY(unsigned char *dst, unsigned char *src, size_t n) {
     while(src<e) *(dst++)=*(src++);
 }
 #endif
-#define VM      GM
 
 int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 {
@@ -310,7 +309,7 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 			if((x & 3) + pixel > 3)
 			    first++;
 			for (offset = first; pixel < length; offset++) {
-			    gr_writeb(colors[pixel], offset);
+			    *(GM+offset) = colors[pixel];
 			    pixel += 4;
 			}
 		    }
@@ -334,7 +333,7 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 		        vga_setpage(segment);
 		        MEMCPY(GM + offset, colors, free);
 		        vga_setpage(segment + 1);
-		        MEMCPY(VM, colors + free, length - free);
+		        MEMCPY(GM, colors + free, length - free);
 		    } else {
 		        vga_setpage(segment);
 		        MEMCPY(GM + offset, colors, length);
@@ -399,11 +398,11 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
     
 		    switch (i) {
 		    case 2:
-		        gr_writeb(colors[2], 0xfffe);
-		        gr_writeb(colors[2], 0xffff);
+                        *(GM+0xfffe) = colors[2];
+			*(GM+0xffff) = colors[1];
 		        break;
 		    case 1:
-		        gr_writeb(colors[2], 0xffff);
+			*(GM+0xffff) = colors[2];
 		        break;
 		    }
     
@@ -411,14 +410,14 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
     
 		    switch (i) {
 		    case 1:
-		        gr_writeb(colors[1], 0);
-		        gr_writeb(colors[0], 1);
+                        *(GM+1) = colors[0];
+			*(GM)   = colors[1];
 		        i = 3 - i;
 		        free += i;
 		        colors += 3;
 		        break;
 		    case 2:
-		        gr_writeb(colors[0], 0);
+                        *(GM) = colors[0];
 		        i = 3 - i;
 		        free += i;
 		        colors += 3;
